<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Zakeke_Webservice {

	/**
	 * Zakeke integration settings.
	 *
	 * @var Zakeke_Integration
	 */
	private $integration;

	/**
	 * Debug mode.
	 *
	 * @var boolean
	 */
	private $debug;

	/**
	 * Logger instance.
	 *
	 * @var WC_Logger
	 */
	private $logger;

	/**
	 * Setup class.
	 */
	public function __construct() {
		$this->integration = new Zakeke_Integration();
		$this->logger      = new WC_Logger();
	}

	/**
	 * Performs the underlying HTTP request.
	 *
	 * @param  string $method HTTP method (GET|POST|PUT|PATCH|DELETE)
	 * @param  string $resource MailChimp API resource to be called
	 * @param  array $args array of parameters to be passed
	 * @param string $auth_token Authentication token
	 *
	 * @throws Exception
	 * @return array          array of decoded result
	 */
	private function request( $method, $resource, $args = array(), $auth_token = null ) {
		$url = ZAKEKE_WEBSERVICE_URL . $resource;

		global $wp_version;

		$request_args = array(
			'method'      => $method,
			'redirection' => 5,
			'httpversion' => '1.1',
			'headers'     => array(
				'Content-Type' => 'application/json',
				'Accept'       => 'application/json',
				'User-Agent'   => 'woocommerce-zakeke/' . ZAKEKE_VERSION . '; WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )
			),
		);

		if ( ! is_null( $auth_token ) ) {
			$request_args['headers']['X-Auth-Token'] = $auth_token;
		}

		// attach arguments (in body or URL)
        if ( ! empty( $args ) ) {
            if ($method === 'GET') {
                $url = $url . '?' . http_build_query($args);
            } else {
                $request_args['body'] = json_encode($args);
            }
        }

		$raw_response = wp_remote_request( $url, $request_args );

		$this->maybe_log( $url, $method, $args, $raw_response );

		if ( is_wp_error( $raw_response )
		     || ( is_array( $raw_response )
		          && $raw_response['response']['code']
		          && floor( $raw_response['response']['code'] ) / 100 >= 4 )
		) {
			throw new Exception( 'Zakeke_Webservice::request ' . $resource . ' ' . print_r( $raw_response, true ) );
		}

		$json   = wp_remote_retrieve_body( $raw_response );
		$result = json_decode( $json, true );

		return $result;
	}

	/**
	 * Get the minimal data required to get the authentication token
	 *
	 * @return array
	 */
	private function auth_minimal_data() {
		$data = array(
			'user' => $this->integration->get_option( 'username' ),
			'pwd'  => $this->integration->get_option( 'password' )
		);

		return $data;
	}

	/**
	 * Zakeke authentication token.
	 *
	 * @param array $data
	 *
	 * @throws Exception
	 * @return string
	 */
	public function auth_token( $data ) {
		$data = array_merge( $data, $this->auth_minimal_data() );
		$res  = $this->request( 'GET', '/api/Login', $data );

		return $res['token'];
	}

    /**
     * Check Zakeke authentication credentials.
     *
     * @param string $username Zakeke username
     * @param string $password Zakeke password
     *
     * @throws Exception
     * @return bool
     */
    public function are_valid_credentials( $username, $password ) {
        try {
            $this->request('GET', '/api/Login', array(
                'user' => $username,
                'pwd'  => $password
            ));

            return true;
        } catch ( Exception $e ) {
            return false;
        }
    }

	/**
	 * Associate the guest with a customer
	 *
	 * @param string $guest_code - Guest identifier.
	 * @param string $customer_id - Customer identifier.
	 *
	 * @throws Exception
	 * @return void
	 */
	public function associate_guest( $guest_code, $customer_id ) {
		$data       = self::auth_minimal_data();
		$data['vc'] = $guest_code;
		$data['cc'] = $customer_id;

		self::auth_token( $data );
	}

	/**
	 * Get the needed data for adding a product to the cart
	 *
	 * @param string $designId Zakeke design identifier.
	 * @param int $qty Quantity.
	 *
	 * @throws Exception
	 * @return object
	 */
	public function cart_info( $designId, $qty ) {
		$data = self::auth_minimal_data();
		$data['qty'] = $qty;

		$resource = '/api/designdocs/' . $designId . '/cartinfo';

		$json = self::request( 'GET', $resource, $data );

		$res = new stdClass();

		$res->pricing = $json['pricing'];

		$preview        = new stdClass();
		$preview->url   = $json['tempPreviewUrl'];
		$preview->label = '';

		$res->previews = array($preview);

		return $res;
	}

	/**
	 * Order containing Zakeke customized products placed.
	 *
	 * @param array $data The data of the order.
	 *
	 * @throws Exception
	 * @return void
	 */
	public function place_order( $data ) {
		$auth_data = self::auth_minimal_data();
		if ( isset( $data['customerID'] ) ) {
			$auth_data['cc'] = $data['customerID'];
		} elseif ( isset( $data['visitorID'] ) ) {
			$auth_data['vc'] = $data['visitorID'];
		}

		$auth_token = $this->auth_token( $auth_data );

		$data['marketplaceID'] = '1';

		self::request( 'POST', '/api/orderdocs', $data, $auth_token );
	}

	/**
	 * Get the Zakeke design preview files
	 *
	 * @param string $designId Zakeke design identifier.
	 *
	 * @throws Exception
	 * @return array
	 */
	public function get_previews( $designId ) {
		$auth_token = self::auth_token( self::auth_minimal_data() );

		$data = array(
			'docid' => $designId
		);

		$json = self::request(
			'GET',
			'/api/designs/0/previewfiles',
			$data,
			$auth_token
		);

		$previews = array();
		foreach ( $json as $preview ) {
			if ( $preview['format'] == 'SVG' ) {
				continue;
			}

			$previewObj        = new stdClass();
			$previewObj->url   = $preview['url'];
			$previewObj->label = $preview['sideName'];
			$previews[]        = $previewObj;
		}

		return $previews;
	}

	/**
	 * Get the Zakeke design output zip
	 *
	 * @param string $designId Zakeke design identifier.
	 *
	 * @throws Exception
	 * @return string
	 */
	public function get_zakeke_output_zip( $designId ) {
		$auth_token = self::auth_token( self::auth_minimal_data() );

		$data = array(
			'docid' => $designId
		);
		$json = self::request( 'GET', '/api/designs/0/outputfiles/zip', $data,
			$auth_token );

		return $json['url'];
	}

	/**
	 * Conditionally log Zakeke Webservice Call
	 *
	 * @param  string $url Zakeke url.
	 * @param  string $method HTTP Method.
	 * @param  array $args HTTP Request Body.
	 * @param  array $response WP HTTP Response.
	 *
	 * @return void
	 */
	private function maybe_log( $url, $method, $args, $response ) {
		if ( ! $this->debug ) {
			return;
		}

		$this->logger->add( 'zakeke', "Zakeke Webservice Call URL: $url \n METHOD: $method \n BODY: " . print_r( $args,
				true ) . ' \n RESPONSE: ' . print_r( $response, true ) );
	}
}